@@ -82,6 +82,14 @@ class Properties:
         weight = self.xtl.Structure.weight()/fg.Na # g
         return weight/vol
 
+    def atoms_per_volume(self):
+        """Return no. atoms per volume in atoms per A^3"""
+
+        occ = self.xtl.Structure.occupancy
+        natoms = np.sum(occ)
+        vol = self.volume()  # A^3
+        return natoms / vol
+
     def weight(self):
         """Return the molecular weight in g/mol"""
         return self.xtl.Structure.weight()
@@ -574,9 +582,10 @@ class Properties:
         out += self.xtl.Cell.info()
 
         if energy_kev is not None:
-            out += '\nEnergy = %s\n' % energy_kev
+            out += '\n    Energy = %7.4f keV\n' % energy_kev
+            out += 'Wavelength = %6.3f A\n' % fc.energy2wave(energy_kev)
             # Xray edges
-            out += '  Near X-Ray Edges:\n'
+            out += '\nNear X-Ray Edges:\n'
             for edge_str, edge_en in zip(*self.xray_edges()):
                 if abs(edge_en - energy_kev) < 1:
                     out += '    %5s : %7.4f keV\n' % (edge_str, edge_en)
