@@ -16,8 +16,8 @@ Usage:
 All plots generated require plt.show() call, unless using interactive mode
 
 
-Version 1.8
-Last updated: 03/04/20
+Version 2.0
+Last updated: 27/09/21
 
 Version History:
 06/01/18 1.0    Program created from DansGeneralProgs.py V2.3
@@ -29,6 +29,9 @@ Version History:
 22/03/19 1.6    Added plot_circle, updated vecplot and plot_lattice_lines
 03/05/19 1.7    Added legend to labels function
 09/03/20 1.8    Added default figure size
+22/11/20 1.9    Update to labels to fix 3D plotting
+15/02/21 1.9.1  Update to vecplot for plotting on axis
+27/09/21 2.0    Added plot3darray, plot_diffractometer_reciprocal_space
 
 @author: DGPorter
 """
@@ -43,7 +46,7 @@ from mpl_toolkits.mplot3d import proj3d
 from . import functions_general as fg
 from . import functions_crystallography as fc
 
-__version__ = '1.8'
+__version__ = '1.9'
 
 DEFAULT_FONT = 'Times New Roman'
 FIGURE_SIZE = [12, 10]
@@ -80,9 +83,9 @@ def labels(ttl=None, xvar=None, yvar=None, zvar=None, legend=False, size='Normal
     plt.xticks(fontsize=tik, fontname=font)
     plt.yticks(fontsize=tik, fontname=font)
     plt.setp(plt.gca().spines.values(), linewidth=2)
-    if plt.gca().get_yaxis().get_scale() != 'log':
+    if plt.gca().get_yaxis().get_scale() != 'log' and 'linear' in plt.gca().name:
         plt.ticklabel_format(useOffset=False)
-        plt.ticklabel_format(style='sci',scilimits=(-3,3))
+        plt.ticklabel_format(style='sci', scilimits=(-3,3))
 
     if ttl is not None:
         plt.gca().set_title(ttl, fontsize=tit, fontweight='bold', fontname=font)
@@ -96,6 +99,9 @@ def labels(ttl=None, xvar=None, yvar=None, zvar=None, legend=False, size='Normal
     if zvar is not None:
         # Don't think this works, use ax.set_zaxis
         plt.gca().set_zlabel(zvar, fontsize=lab, fontname=font)
+        for t in plt.gca().zaxis.get_major_ticks():
+            t.label.set_fontsize(tik)
+            t.label.set_fontname(font)
 
     if legend:
         plt.legend(loc=0, frameon=False, prop={'size': leg, 'family': 'serif'})
@@ -224,6 +230,7 @@ def newplot3(*args, **kwargs):
     Shortcut to creating a simple 3D plot
     Automatically tiles 1 dimensional x and y arrays to match 2D z array,
     assuming z.shape = (len(x),len(y))
+      newplot3(x, y, z, ...)
 
     E.G.
       newplot3([1,2,3,4],[9,8,7],[[2,4,6],[8,10,12],[14,16,18],[20,22,24]],'-o')
@@ -252,6 +259,19 @@ def newplot3(*args, **kwargs):
         ax.plot(*args, **kwargs)
 
 
+def plot3darray(vec, *args, **kwargs):
+    """
+    Plot 3D vectors in 3D
+        plt.plot(vec[:, 0], vec[:, 1], vec[:, 3], *args, **kwargs)
+    :param vec: [n*3] array
+    :param args: args to pass to plt.plot
+    :param kwargs: kwargs to pass to plt.plot
+    :return: matplotlib plot object
+    """
+    vec = np.reshape(vec, (-1, 3))
+    return plt.plot(vec[:, 0], vec[:, 1], vec[:, 2], *args, **kwargs)
+
+
 def sliderplot(YY, X=None, slidervals=None, *args, **kwargs):
     """
     Shortcut to creating a simple 2D plot with a slider to go through a third dimension
@@ -368,7 +388,7 @@ def sliderplot2D(ZZZ, XX=None, YY=None, slidervals=None, *args, **kwargs):
     sldr.on_changed(update)
 
 
-def plot_cell(cell_centre=[0, 0, 0], CELL=np.eye(3)):
+def plot_cell(cell_centre=[0, 0, 0], CELL=np.eye(3), color='k'):
     """
     Plot a box defined by a unit cell on the current plot
     :param cell_centre: [1x3] array : centre of cell, default [0,0,0]
@@ -376,12 +396,12 @@ def plot_cell(cell_centre=[0, 0, 0], CELL=np.eye(3)):
     :return: None
     """
 
-    uvw = np.array([[0., 0, 0], [1, 0, 0], [1, 0, 1], [1, 1, 1], [1, 1, 0], [0, 1, 0], [0, 1, 1], \
+    uvw = np.array([[0., 0, 0], [1, 0, 0], [1, 0, 1], [1, 1, 1], [1, 1, 0], [0, 1, 0], [0, 1, 1],
                     [0, 0, 1], [1, 0, 1], [1, 0, 0], [1, 1, 0], [1, 1, 1], [0, 1, 1], [0, 1, 0], [0, 0, 0], [0, 0, 1]])
     uvw = uvw - 0.5  # plot around box centre
     bpos = np.dot(uvw, CELL)
     bpos = bpos + cell_centre
-    plt.plot(bpos[:, 0], bpos[:, 1], bpos[:, 2], c='k')  # cell box
+    plt.plot(bpos[:, 0], bpos[:, 1], bpos[:, 2], c=color)  # cell box
 
 
 def plot_circle(radius=1.0, centre=[0,0], height=0, *args, **kwargs):
@@ -493,18 +513,37 @@ def vecplot(UV, mode='hk0', axis=None, *args, **kwargs):
     plot_lattice_lines(latt, UV[0], UV[1], axis=axis, *args, **kwargs)
 
 
-def UV_arrows(UV):
+def UV_arrows(UV, alabel='a', blabel='b', clabel='c'):
     """
     Plot arrows with a*,b* on current figure
     """
     # Get current axis size
     ax = plt.gca()
+    if ax.name.lower() == '3d':
+        # 3D plot
+        ax_xlim = ax.get_xlim()
+        ax_ylim = ax.get_ylim()
+        ax_zlim = ax.get_zlim()
+        arrow_size = 40
+        color = 'k'
+        fontsize = 18
+        plot_arrow([0, UV[0, 0]], [0, UV[0, 1]], [0, UV[0, 2]], arrow_size=arrow_size, col=color)
+        ax.text(UV[0, 0], UV[0, 1], UV[0, 2], alabel, fontname=DEFAULT_FONT, weight='bold', size=fontsize)
+        plot_arrow([0, UV[1, 0]], [0, UV[1, 1]], [0, UV[1, 2]], arrow_size=arrow_size, col=color)
+        ax.text(UV[1, 0], UV[1, 1], UV[1, 2], blabel, fontname=DEFAULT_FONT, weight='bold', size=fontsize)
+        plot_arrow([0, UV[2, 0]], [0, UV[2, 1]], [0, UV[2, 2]], arrow_size=arrow_size, col=color)
+        ax.text(UV[2, 0], UV[2, 1], UV[2, 2], clabel, fontname=DEFAULT_FONT, weight='bold', size=fontsize)
+        ax.set_xlim(ax_xlim)
+        ax.set_ylim(ax_ylim)
+        ax.set_zlim(ax_zlim)
+        return
+    # 2D plot
     axsize = ax.axis()
     asty = dict(arrowstyle="->")
     plt.annotate("", xy=(UV[0, 0], UV[0, 1]), xytext=(0.0, 0.0), arrowprops=asty)
     plt.annotate("", xy=(UV[1, 0], UV[1, 1]), xytext=(0.0, 0.0), arrowprops=asty)
-    plt.annotate("a", (0.1 + UV[0, 0], UV[0, 1] - 0.2))
-    plt.annotate("b", (UV[1, 0] - 0.2, 0.1 + UV[1, 1]))
+    plt.annotate(alabel, (0.1 + UV[0, 0], UV[0, 1] - 0.2))
+    plt.annotate(blabel, (UV[1, 0] - 0.2, 0.1 + UV[1, 1]))
     ax.axis(axsize)
 
 
@@ -555,7 +594,7 @@ def plot_lattice_points2D(Q, markersize=12, color='b', marker='o'):
     ax.axis(axsize)
 
 
-def plot_lattice_lines(latt, vec_a=[1, 0, 0], vec_b=[0, 1, 0], axis=None, *args, **kwargs):
+def plot_lattice_lines(latt, vec_a=(1, 0, 0), vec_b=(0, 1, 0), axis=None, *args, **kwargs):
     """
     Add lines defining the reciprocal lattice to the current plot
         Generates square or hexagonal lines where vertices are the lattice points within the image.
@@ -600,7 +639,7 @@ def plot_lattice_lines(latt, vec_a=[1, 0, 0], vec_b=[0, 1, 0], axis=None, *args,
 
 
 def plot_vector_arrows(vec_a=[1, 0, 0], vec_b=[1, 0, 0], vec_a_lab=None, vec_b_lab=None,
-                       arrow_size=40, color='b', fontsize=18):
+                       arrow_size=40, color='b', fontsize=18, axis=None):
     """
     Plot vector arrows for Cell on current axis
         Will generate two arrows on the current axis, pointing from the origin to vec_a and vec_b, respectivley.
@@ -617,8 +656,9 @@ def plot_vector_arrows(vec_a=[1, 0, 0], vec_b=[1, 0, 0], vec_a_lab=None, vec_b_l
     vec_a = np.asarray(vec_a).reshape([-1, np.shape(vec_a)[-1]])
     vec_b = np.asarray(vec_b).reshape((-1, np.shape(vec_b)[-1]))
 
-    ax = plt.gca()
-    axsize = ax.axis()
+    if axis is None:
+        axis = plt.gca()
+    axsize = axis.axis()
 
     # Vector arrows and lattice point labels
     if vec_a_lab is None:
@@ -626,11 +666,12 @@ def plot_vector_arrows(vec_a=[1, 0, 0], vec_b=[1, 0, 0], vec_a_lab=None, vec_b_l
     if vec_b_lab is None:
         vec_b_lab = 'b*'
 
+    plt.sca(axis)
     plot_arrow([0, vec_a[0, 0]], [0, vec_a[0, 1]], arrow_size=arrow_size, col=color)
     plt.text(vec_a[0, 0], vec_a[0, 1], vec_a_lab, fontname=DEFAULT_FONT, weight='bold', size=fontsize)
     plot_arrow([0, vec_b[0, 0]], [0, vec_b[0, 1]], arrow_size=arrow_size, col=color)
     plt.text(vec_b[0, 0], vec_b[0, 1], vec_b_lab, fontname=DEFAULT_FONT, weight='bold', size=fontsize)
-    ax.axis(axsize)
+    axis.axis(axsize)
 
 
 def plot_ewald_coverage(energy_kev, color='k', linewidth=2):
@@ -639,6 +680,8 @@ def plot_ewald_coverage(energy_kev, color='k', linewidth=2):
     Includes boundaries for theta=0, twotheta=180 and theta=twotheta
 
     :param energy_kev: float
+    :param color: str
+    :param linewidth: float
     :return: None
     """
 
@@ -657,6 +700,55 @@ def plot_ewald_coverage(energy_kev, color='k', linewidth=2):
     plt.axis([-q_max, q_max, 0, q_max])
 
 
+def plot_diffractometer_reciprocal_space(phi, chi, eta, mu, delta, gamma, uv, u, lab, energy_kev):
+    """
+    Plot crystal axes in lab frame of 6-circle diffractometer
+    :param phi:
+    :param chi:
+    :param eta:
+    :param mu:
+    :param delta:
+    :param gamma:
+    :param uv:
+    :param u:
+    :param lab:
+    :param energy_kev:
+    :return:
+    """
+
+    uvstar = fc.RcSp(uv)
+    maxhkl = fc.maxHKL(2, uvstar)
+    hkl = fc.genHKL(*maxhkl)
+    r = fc.diffractometer_rotation(phi, chi, eta, mu)
+    qdet = fc.diff6circleq(delta, gamma, energy_kev, lab=lab)
+    ki, kf = fc.diff6circlek(delta, gamma, energy_kev, lab=lab)
+    qlab = fc.labwavevector(hkl, uv, u, r, lab)
+    astar = fc.labwavevector([1, 0, 0], uv, u, r, lab)
+    bstar = fc.labwavevector([0, 1, 0], uv, u, r, lab)
+    cstar = fc.labwavevector([0, 0, 1], uv, u, r, lab)
+
+    fig = plt.figure(figsize=FIGURE_SIZE, dpi=FIGURE_DPI)
+    ax = fig.add_subplot(111, projection='3d')
+
+    def pltvec(vec, *args, **kwargs):
+        vec = np.reshape(vec, (-1, 3))
+        return plt.plot(vec[:, 1], vec[:, 2], vec[:, 0], *args, **kwargs)
+
+    pltvec(qlab, 'r+', ms=12, label='hkl')
+    pltvec([-ki, [0, 0, 0], kf, [0, 0, 0], qdet], 'k-', lw=5, label='q = kf - ki')
+    pltvec([[0, 0, 0], qdet], 'm-', lw=5, label='q = kf - ki')
+    pltvec([[0, 0, 0], astar], 'b-', lw=5, label='astar')
+    pltvec([[0, 0, 0], bstar], 'g-', lw=5, label='bstar')
+    pltvec([[0, 0, 0], cstar], 'y-', lw=5, label='cstar')
+    labels(None, 'Y', 'Z', 'X', legend=True)
+    ax.set_xlim([2, -2])
+    ax.set_ylim([2, -2])
+    ax.set_zlim([-2, 2])
+    #ax.invert_xaxis()
+    #ax.invert_yaxis()
+    plt.show()
+
+
 def plot_xray_scattering_factor(elements, maxq=10):
     """
     Plot x-ray scattering factor for 1 or more elements
@@ -705,9 +797,9 @@ def plot_xray_attenuation(elements, min_energy=0, max_energy=20):
     labels('X-Ray Attenuation', 'Energy [keV]', r'$\mu/\rho$ [cm$^2$/g]')
 
 
-def plot_xray_scattering_factor(element, min_energy=0.5, max_energy=20):
+def plot_atomic_scattering_factor(element, min_energy=0.5, max_energy=20):
     """
-    Plot x-ray scattering factor for 1 or more elements
+    Plot atomic scattering factor for 1 or more elements
     :param element: str name of element to plot
     :param min_energy: float min energy in keV
     :param max_energy: float max energy in keV
@@ -722,3 +814,69 @@ def plot_xray_scattering_factor(element, min_energy=0.5, max_energy=20):
     plt.xlim([min_energy, max_energy])
     labels('X-Ray Scattering Factor\n%s' % element, 'Energy [keV]', None, legend=True)
 
+
+def plot_xray_transmission(chemical_formula, density=8.9, energy_range=None, thickness_um=100):
+    """
+    Plot transmission of x-ray through a slab of material at range of energies
+    Equivalent to https://henke.lbl.gov/optical_constants/filter2.html
+    Based on formulas from: Henke, Gullikson, and Davis, Atomic Data and Nuclear Data Tables 54 no.2, 181-342 (July 1993)
+    :param chemical_formula: str molecular formula
+    :param density: float density in g/cm^3
+    :param energy_range: array x-ray energy in keV, None for default range
+    :param thickness_um: slab thickness in microns
+    :return: float or array
+    """
+    if energy_range is None:
+        energy_range = np.arange(0.03, 20, 0.01)
+
+    transmission = fc.filter_transmission(chemical_formula, energy_range, density, thickness_um)
+    ttl = '%s Density=%5.3f, thickness=%3.3g μm' % (chemical_formula, density, thickness_um)
+
+    newplot(energy_range, transmission)
+    labels(ttl, 'Energy [keV]', 'Transmission')
+
+
+def plot_xray_attenuation_length(chemical_formula, density=8.9, energy_range=None, grazing_angle=90):
+    """
+    Plot the X-Ray Attenuation Length of a compound
+    Equivalent to: https://henke.lbl.gov/optical_constants/atten2.html
+    Based on formulas from: Henke, Gullikson, and Davis, Atomic Data and Nuclear Data Tables 54 no.2, 181-342 (July 1993)
+    :param chemical_formula: str molecular formula
+    :param density: float density in g/cm^3
+    :param energy_range: array x-ray energy in keV, None for default range
+    :param grazing_angle: incidence angle relative to the surface, in degrees
+    :return: float or array, in microns
+    """
+    if energy_range is None:
+        energy_range = np.arange(0.03, 20, 0.01)
+
+    transmission = fc.molecular_attenuation_length(chemical_formula, energy_range, density, grazing_angle)
+    ttl = '%s Density=%5.3f, Angle=%3.3g deg' % (chemical_formula, density, grazing_angle)
+
+    newplot(energy_range, transmission)
+    labels(ttl, 'Energy [keV]', 'Atten Length [μm]')
+
+
+def plot_xray_refractive_index(chemical_formula, density=8.9, energy_range=None):
+    """
+    Plot the Complex Index of Refraction of a compound
+        n = 1 - (1/2pi)N*r0*lambda^2*(f1+if2) = 1 - Delta - iBeta
+    Equivalent to: https://henke.lbl.gov/optical_constants/getdb2.html
+    Based on formulas from: Henke, Gullikson, and Davis, Atomic Data and Nuclear Data Tables 54 no.2, 181-342 (July 1993)
+    :param chemical_formula: str molecular formula
+    :param density: float density in g/cm^3
+    :param energy_range: array x-ray energy in keV, None for default range
+    :return: float or array, in microns
+    """
+    if energy_range is None:
+        energy_range = np.arange(0.03, 20, 0.01)
+
+    n, delta, beta = fc.molecular_refractive_index(chemical_formula, energy_range, density)
+    ttl = '%s Density=%5.3f\nIndex of Refraction = 1 - δ - iβ' % (chemical_formula, density)
+
+    newplot(energy_range, delta, 'r-', lw=2, label='δ')
+    plt.plot(energy_range, beta, 'b-', lw=2, label='β')
+    labels(ttl, 'Energy [keV]', None, legend=True)
+    plt.xscale('log')
+    plt.yscale('log')
+
