@@ -5,6 +5,8 @@ Main crystal gui windows
 import sys, os
 import matplotlib.pyplot as plt # Plotting
 import numpy as np
+
+
 if sys.version_info[0] < 3:
     import Tkinter as tk
     import tkFileDialog as filedialog
@@ -18,14 +20,12 @@ else:
 from ..classes_crystal import Crystal
 from ..classes_structures import Structures
 from ..classes_fdmnes import fdmnes_checker
-from .. import functions_general as fg
-from .. import functions_plotting as fp
 from .. import functions_crystallography as fc
 from .basic_widgets import StringViewer, SelectionBox
-from .basic_widgets import (TF, BF, SF, LF, HF, TTF,
-                            bkg, ety, btn, opt, btn2,
-                            btn_active, opt_active, txtcol,
-                            btn_txt, ety_txt, opt_txt)
+from .basic_widgets import (TF, BF, SF, HF, TTF,
+                            bkg, ety, btn, opt, btn_active, opt_active, txtcol,
+                            ety_txt)
+from .properties import PropertiesGui
 from .scattering import ScatteringGui
 from .multi_crystal import MultiCrystalGui
 from .multiple_scattering import MultipleScatteringGui
@@ -748,7 +748,6 @@ class SymmetryGui:
             self.text_1.insert(tk.END, '%3.0f\n' % n)
             sym = self.Symmetry.symmetry_operations[n].strip('\"\'')
             mag = self.Symmetry.symmetry_operations_magnetic[n]
-            mag = mag.strip('\"\'').replace('x', 'mx').replace('y', 'my').replace('z', 'mz')
             symstr += '%25s\n' % (sym)
             magstr += '%25s\n' % (mag)
 
@@ -828,261 +827,6 @@ class SymmetryGui:
         self.root.destroy()
 
 
-class PropertiesGui:
-    """
-    Show properties of atomis in Crystal
-    """
-
-    def __init__(self, xtl):
-        """Initialise"""
-        self.xtl = xtl
-        # Create Tk inter instance
-        self.root = tk.Tk()
-        self.root.wm_title('Properties %s' % xtl.name)
-        # self.root.minsize(width=640, height=480)
-        self.root.maxsize(width=self.root.winfo_screenwidth(), height=self.root.winfo_screenheight())
-        self.root.tk_setPalette(
-            background=bkg,
-            foreground=txtcol,
-            activeBackground=opt_active,
-            activeForeground=txtcol)
-
-        frame = tk.Frame(self.root)
-        frame.pack(side=tk.LEFT, anchor=tk.N)
-
-        # Crystal Atoms
-        atoms = np.unique(xtl.Atoms.type)
-        atoms = fc.arrange_atom_order(atoms)
-        elements = fc.atom_properties(None, 'Element')
-
-        # Variables
-        self.zfraction = tk.DoubleVar(frame, 1)
-        self.atoms = tk.StringVar(frame, ' '.join(atoms))
-        self.atomopt = tk.StringVar(frame, 'Elements:')
-        self.energy_kev = tk.DoubleVar(frame, 8.0)
-        self.wavelength = tk.DoubleVar(frame, 1.5498)
-        self.edge = tk.StringVar(frame, 'Edge')
-        self.twotheta = tk.DoubleVar(frame, 90.0)
-        self.qmag = tk.DoubleVar(frame, 5.733)
-        self.dspace = tk.DoubleVar(frame, 1.096)
-
-        # X-ray edges:
-        self.xr_edges, self.xr_energies = self.xtl.Properties.xray_edges()
-        self.xr_edges.insert(0, 'Cu Ka')
-        self.xr_edges.insert(1, 'Mo Ka')
-        self.xr_energies.insert(0, fg.Cu)
-        self.xr_energies.insert(1, fg.Mo)
-
-        # ---Line 0---
-        line = tk.Frame(frame)
-        line.pack(side=tk.TOP, expand=tk.TRUE, pady=5)
-
-        # Cell Name
-        var = tk.Label(line, text=xtl.Properties.molname(), font=TF)
-        var.pack(side=tk.LEFT)
-        var = tk.Label(line, text=xtl.Properties.molcharge(), font=TF)
-        var.pack(side=tk.LEFT, padx=15)
-
-        # ---Line 1---
-        line = tk.Frame(frame)
-        line.pack(side=tk.TOP, expand=tk.TRUE, pady=5)
-
-        # Cell properties
-        var = tk.Label(line, text='Weight = %8.2f g/mol' % xtl.Properties.weight(), font=TF)
-        var.pack(side=tk.LEFT)
-        var = tk.Label(line, text='Volume = %8.2f A^3' % xtl.Properties.volume(), font=TF)
-        var.pack(side=tk.LEFT)
-        var = tk.Label(line, text='Density = %8.2f g/cm' % xtl.Properties.density(), font=TF)
-        var.pack(side=tk.LEFT)
-
-        # ---Line 2---
-        line = tk.Frame(frame)
-        line.pack(side=tk.TOP, expand=tk.TRUE, pady=5)
-
-        # Energy wavelength Conversions
-        var = tk.Label(line, text='Energy:', font=TF)
-        var.pack(side=tk.LEFT)
-        var = tk.Entry(line, textvariable=self.energy_kev, font=TF, width=16, bg=ety, fg=ety_txt)
-        var.pack(side=tk.LEFT)
-        var.bind('<Return>', self.fun_energy2wave)
-        var.bind('<KP_Enter>', self.fun_energy2wave)
-        var = tk.Label(line, text='keV <-> Wavelength:', font=TF)
-        var.pack(side=tk.LEFT)
-        var = tk.Entry(line, textvariable=self.wavelength, font=TF, width=16, bg=ety, fg=ety_txt)
-        var.pack(side=tk.LEFT)
-        var.bind('<Return>', self.fun_wave2energy)
-        var.bind('<KP_Enter>', self.fun_wave2energy)
-        var = tk.Label(line, text='A', font=TF)
-        var.pack(side=tk.LEFT)
-        var = tk.OptionMenu(line, self.edge, *self.xr_edges, command=self.fun_edge)
-        var.config(font=SF, width=5, bg=opt, activebackground=opt_active)
-        var["menu"].config(bg=opt, bd=0, activebackground=opt_active)
-        var.pack(side=tk.LEFT)
-
-        # ---Line 3---
-        line = tk.Frame(frame)
-        line.pack(side=tk.TOP, expand=tk.TRUE, pady=5)
-
-        # Two-Theta - Q - d conversion
-        var = tk.Label(line, text='Two-Theta:', font=TF)
-        var.pack(side=tk.LEFT)
-        var = tk.Entry(line, textvariable=self.twotheta, font=TF, width=10, bg=ety, fg=ety_txt)
-        var.pack(side=tk.LEFT)
-        var.bind('<Return>', self.fun_tth2q)
-        var.bind('<KP_Enter>', self.fun_tth2q)
-        var = tk.Label(line, text='Deg <-> Q:', font=TF)
-        var.pack(side=tk.LEFT)
-        var = tk.Entry(line, textvariable=self.qmag, font=TF, width=10, bg=ety, fg=ety_txt)
-        var.pack(side=tk.LEFT)
-        var.bind('<Return>', self.fun_q2tth)
-        var.bind('<KP_Enter>', self.fun_q2tth)
-        var = tk.Label(line, text='A^-1 <-> d-spacing:', font=TF)
-        var.pack(side=tk.LEFT)
-        var = tk.Entry(line, textvariable=self.dspace, font=TF, width=10, bg=ety, fg=ety_txt)
-        var.pack(side=tk.LEFT)
-        var.bind('<Return>', self.fun_d2tth)
-        var.bind('<KP_Enter>', self.fun_d2tth)
-        var = tk.Label(line, text='A', font=TF)
-        var.pack(side=tk.LEFT)
-
-        # ---Line 3---
-        line = tk.Frame(frame)
-        line.pack(side=tk.TOP, pady=5)
-
-        var = tk.Label(line, text='Elements', font=LF)
-        var.pack(side=tk.LEFT)
-
-        var = tk.Entry(line, textvariable=self.atoms, font=TF, width=16, bg=ety, fg=ety_txt)
-        var.pack(side=tk.LEFT)
-
-        var = tk.OptionMenu(line, self.atomopt, *elements, command=self.fun_element)
-        var.config(font=SF, width=10, bg=opt, activebackground=opt_active)
-        var["menu"].config(bg=opt, bd=0, activebackground=opt_active)
-        var.pack(side=tk.LEFT)
-
-        var = tk.Button(line, text='Mass Fraction', font=BF, command=self.fun_frac, bg=btn, activebackground=btn_active)
-        var.pack(side=tk.LEFT)
-        var = tk.Label(line, text=' Z:', font=TF)
-        var.pack(side=tk.LEFT)
-        var = tk.Entry(line, textvariable=self.zfraction, font=TF, width=2, bg=ety, fg=ety_txt)
-        var.pack(side=tk.LEFT)
-
-        # ---Line 4---
-        line = tk.Frame(frame)
-        line.pack(side=tk.TOP, pady=5)
-
-        var = tk.Button(line, text='Properties', height=2, font=BF, command=self.fun_prop, bg=btn,
-                        activebackground=btn_active)
-        var.pack(side=tk.LEFT)
-
-        var = tk.Button(line, text='Magnetic\nForm Factor', font=BF, command=self.fun_magff, bg=btn,
-                        activebackground=btn_active)
-        var.pack(side=tk.LEFT)
-
-        var = tk.Button(line, text='X-Ray\nAttenuation', font=BF, command=self.fun_xratten, bg=btn,
-                        activebackground=btn_active)
-        var.pack(side=tk.LEFT)
-
-        var = tk.Button(line, text='X-Ray\nScattering Factor', font=BF, command=self.fun_xsf, bg=btn,
-                        activebackground=btn_active)
-        var.pack(side=tk.LEFT)
-
-    def fun_frac(self):
-        """Atomic Fraction"""
-        z = self.zfraction.get()
-        out = self.xtl.Properties.molfraction(z)
-        StringViewer(out, self.xtl.name)
-
-    def fun_energy2wave(self, event=None):
-        """Convert energy to wavelength"""
-        energy = self.energy_kev.get()
-        wavelength = fc.energy2wave(energy)
-        self.wavelength.set(round(wavelength, 4))
-        self.fun_tth2q()
-
-    def fun_wave2energy(self, event=None):
-        """Convert wavelength to energy"""
-        wavelength = self.wavelength.get()
-        energy = fc.wave2energy(wavelength)
-        self.energy_kev.set(round(energy, 5))
-        self.fun_tth2q()
-
-    def fun_edge(self, event=None):
-        """Add edge energy"""
-        edge = self.edge.get()
-        if self.edge.get() in self.xr_edges:
-            idx = self.xr_edges.index(edge)
-            self.energy_kev.set(self.xr_energies[idx])
-            self.fun_energy2wave()
-
-    def fun_tth2q(self, event=None):
-        """Convert two-theta to q and d"""
-        tth = self.twotheta.get()
-        energy = self.energy_kev.get()
-        qmag = fc.calqmag(tth, energy)
-        dspace = fc.q2dspace(qmag)
-        self.qmag.set(round(qmag, 4))
-        self.dspace.set(round(dspace, 4))
-
-    def fun_q2tth(self, event=None):
-        """Convert Q to tth and d"""
-        qmag = self.qmag.get()
-        energy = self.energy_kev.get()
-        tth = fc.cal2theta(qmag, energy)
-        dspace = fc.q2dspace(qmag)
-        self.twotheta.set(round(tth, 4))
-        self.dspace.set(round(dspace, 4))
-
-    def fun_d2tth(self, event=None):
-        """Convert d to tth and q"""
-        dspace = self.dspace.get()
-        energy = self.energy_kev.get()
-        qmag = fc.q2dspace(dspace)
-        tth = fc.cal2theta(qmag, energy)
-        self.qmag.set(round(qmag, 4))
-        self.twotheta.set(round(tth, 4))
-
-    def fun_element(self, event=None):
-        """Dropdown menu"""
-        atom = self.atomopt.get()
-        self.atoms.set(atom)
-
-    def fun_prop(self):
-        """Properties button"""
-        elements = self.atoms.get()
-        elements = elements.replace(',', ' ')
-        elelist = elements.split()
-        out = fc.print_atom_properties(elelist)
-        width = 12 + 12*len(elelist)
-        if width > 120: width=120
-        StringViewer(out, 'Element Properties', width)
-
-    def fun_magff(self):
-        """Properties button"""
-        elements = self.atoms.get()
-        elements = elements.replace(',', ' ')
-        elelist = elements.split()
-        fp.plot_magnetic_form_factor(elelist)
-        plt.show()
-
-    def fun_xratten(self):
-        """Properties button"""
-        elements = self.atoms.get()
-        elements = elements.replace(',', ' ')
-        elelist = elements.split()
-        fp.plot_xray_attenuation(elelist)
-        plt.show()
-
-    def fun_xsf(self):
-        """Properties button"""
-        elements = self.atoms.get()
-        elements = elements.replace(',', ' ')
-        elelist = elements.split()
-        for el in elelist:
-            fp.plot_xray_scattering_factor(el)
-        plt.show()
-
-
 class SuperstructureGui:
     """
     Generate a superstructure
